package de.storchp.fdroidbuildstatus

import android.Manifest
import android.content.pm.PackageManager
import android.os.Build.VERSION
import android.os.Build.VERSION_CODES
import android.os.Bundle
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.preference.Preference
import androidx.preference.PreferenceFragmentCompat
import androidx.preference.TwoStatePreference
import dagger.hilt.android.AndroidEntryPoint
import de.storchp.fdroidbuildstatus.utils.PreferencesService
import javax.inject.Inject

@AndroidEntryPoint
class SettingsActivity : AppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.settings_activity)
        if (savedInstanceState == null) {
            supportFragmentManager
                .beginTransaction()
                .replace(R.id.settings, SettingsFragment())
                .commit()
        }
        val actionBar = supportActionBar
        actionBar?.setDisplayHomeAsUpEnabled(true)
    }

    @AndroidEntryPoint
    class SettingsFragment : PreferenceFragmentCompat() {

        @Inject
        lateinit var preferencesService: PreferencesService

        private var updateCheckPref: TwoStatePreference? = null
        private val requestNotificationPermissionLauncher =
            registerForActivityResult(ActivityResultContracts.RequestPermission()) { isGranted: Boolean? ->
                if (!isGranted!!) {
                    updateCheckPref!!.isChecked = false
                    Toast.makeText(
                        requireContext(),
                        R.string.notification_permission_needed,
                        Toast.LENGTH_SHORT
                    ).show()
                }
            }

        override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
            setPreferencesFromResource(R.xml.root_preferences, rootKey)
            if (VERSION.SDK_INT >= VERSION_CODES.TIRAMISU) {
                updateCheckPref = findPreference(getString(R.string.PREF_UPDATE_CHECK))
                assert(updateCheckPref != null)
                updateCheckPref!!.onPreferenceChangeListener =
                    Preference.OnPreferenceChangeListener { _: Preference?, newValue: Any ->
                        if (java.lang.Boolean.TRUE == newValue && isNotificationPermissionDenied) {
                            requestNotificationPermissionLauncher.launch(Manifest.permission.POST_NOTIFICATIONS)
                        }
                        true
                    }
                if (isNotificationPermissionDenied) {
                    updateCheckPref!!.isChecked = false
                }
            }
            val nightModePref = findPreference<Preference>(getString(R.string.night_mode_key))!!
            nightModePref.onPreferenceChangeListener =
                Preference.OnPreferenceChangeListener { _: Preference?, newValue: Any ->
                    requireActivity().runOnUiThread { preferencesService.applyNightMode((newValue as String).toInt()) }
                    true
                }
        }

        @get:RequiresApi(api = VERSION_CODES.TIRAMISU)
        private val isNotificationPermissionDenied: Boolean
            get() = ContextCompat.checkSelfPermission(
                requireContext(),
                Manifest.permission.POST_NOTIFICATIONS
            ) != PackageManager.PERMISSION_GRANTED
    }
}