package de.storchp.fdroidbuildstatus.utils

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.core.app.NotificationCompat
import de.storchp.fdroidbuildstatus.MainActivity
import de.storchp.fdroidbuildstatus.R


object NotificationUtils {

    private val TAG = NotificationUtils::class.java.simpleName
    private const val CHANNEL_ID = "general"

    private const val NEW_BUILD_NOTIFICATION_ID = 0

    fun createNewBuildNotification(context: Context, text: String) {
        Log.d(TAG, "new notification: $text")
        val notificationManager = getNotificationManager(context)
        if (!notificationManager.areNotificationsEnabled()) {
            Log.w(TAG, "notifications disabled")
            return
        }
        val pIntent = PendingIntent.getActivity(
            context,
            System.currentTimeMillis().toInt(),
            Intent(context, MainActivity::class.java),
            PendingIntent.FLAG_IMMUTABLE
        )
        val builder = NotificationCompat.Builder(context, CHANNEL_ID)
            .setContentTitle(context.getString(R.string.app_name))
            .setContentText(text)
            .setSmallIcon(R.drawable.ic_fdroid_buildstatus)
            .setContentIntent(pIntent)
            .setAutoCancel(true)
        notificationManager.notify(NEW_BUILD_NOTIFICATION_ID, builder.build())
    }

    fun cancelNewBuildNotification(context: Context) {
        getNotificationManager(context).cancel(NEW_BUILD_NOTIFICATION_ID)
    }

    private fun getNotificationManager(context: Context): NotificationManager {
        return context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
    }

    fun areNotificationsEnabled(context: Context): Boolean {
        return getNotificationManager(context).areNotificationsEnabled()
    }

    fun createNotificationChannel(context: Context) {
        val channel = NotificationChannel(
            CHANNEL_ID,
            context.getString(R.string.channel_name),
            NotificationManager.IMPORTANCE_DEFAULT
        )
        channel.description = context.getString(R.string.channel_description)
        channel.enableVibration(true)
        getNotificationManager(context).createNotificationChannel(channel)
    }
}